package com.piengineering.pigamecontroller;

import android.os.Bundle;
import android.app.Activity;
//import android.view.Menu; //interferes with the game buttons
import com.piengineering.pigamecontroller.R;
import android.content.Context;
import android.graphics.Color;
import android.hardware.input.InputManager;
import android.util.Log;
import android.util.SparseArray;
import android.util.SparseIntArray;
import android.view.InputDevice;
import android.view.KeyEvent;
import android.view.MotionEvent;
import android.view.InputDevice.MotionRange;
import android.widget.TextView;

import java.util.List;
//from ...\adt-bundle-windows\sdk\samples\android-17\ApiDemos\src\com\example\android\apis\view\GameControllerInput.java minus the messy gui stuff

public class GameControllerInput extends Activity implements InputManager.InputDeviceListener {

	private static final String TAG = "GameControllerInput";

	private InputManager mInputManager;
	private SparseArray<InputDeviceState> mInputDeviceStates;

	@Override
	protected void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);

		mInputManager = (InputManager)getSystemService(Context.INPUT_SERVICE);
		mInputDeviceStates = new SparseArray<InputDeviceState>();
		setContentView(R.layout.activity_main);

	}

	@Override
	protected void onResume() {
		super.onResume();

		// Register an input device listener to watch when input devices are
		// added, removed or reconfigured.
		mInputManager.registerInputDeviceListener(this, null);

		// Query all input devices.
		// We do this so that we can see them in the log as they are enumerated.
		int[] ids = mInputManager.getInputDeviceIds();
		for (int i = 0; i < ids.length; i++) {
			getInputDeviceState(ids[i]);
		}
	}

	@Override
	protected void onPause() {
		super.onPause();

		// Remove the input device listener when the activity is paused.
		mInputManager.unregisterInputDeviceListener(this);
	}

	@Override
	public void onWindowFocusChanged(boolean hasFocus) {
		super.onWindowFocusChanged(hasFocus);

	}

	/*@Override
	public boolean onCreateOptionsMenu(Menu menu) {
		// Inflate the menu; this adds items to the action bar if it is present.
		getMenuInflater().inflate(R.menu.activity_main, menu);
		return true;
	}*/

	@Override
	public boolean dispatchKeyEvent(KeyEvent event) {
		//is it a P.I. Engineering device
		//InputDevice temp=InputDevice.getDevice(event.getDeviceId());
		//if(temp.getName().contains("P. I. Engineering")){

			// Update device state for visualization and logging.
			InputDeviceState state = getInputDeviceState(event.getDeviceId());
			if (state != null) {
				switch (event.getAction()) {
				case KeyEvent.ACTION_DOWN:
					if (state.onKeyDown(event)) {
						show(state);

					}
					break;
				case KeyEvent.ACTION_UP:
					if (state.onKeyUp(event)) {
						// mSummaryAdapter.show(state);
						show(state);

					}
					break;
				}
			}
		//}
		return super.dispatchKeyEvent(event);
	}

	@Override
	public boolean dispatchGenericMotionEvent(MotionEvent event) {
		// Check that the event came from a joystick since a generic motion event
		// could be almost anything.
		if (isJoystick(event.getSource())
				&& event.getAction() == MotionEvent.ACTION_MOVE) {
			//now see if it is a P.I. Engineering device
			//InputDevice temp=InputDevice.getDevice(event.getDeviceId());
			//if(temp.getName().contains("P. I. Engineering")){
				// Update device state for visualization and logging.
				InputDeviceState state = getInputDeviceState(event.getDeviceId());
				if (state != null && state.onJoystickMotion(event)) {

					show(state);
				}
			//}
		}
		return super.dispatchGenericMotionEvent(event);
	}

	private InputDeviceState getInputDeviceState(int deviceId) {
		InputDeviceState state = mInputDeviceStates.get(deviceId);
		if (state == null) {
			final InputDevice device = mInputManager.getInputDevice(deviceId);
			if (device == null) {
				return null;
			}
			state = new InputDeviceState(device);
			mInputDeviceStates.put(deviceId, state);
			Log.i(TAG, "Device enumerated: " + state.mDevice);
		}
		return state;
	}

	// Implementation of InputManager.InputDeviceListener.onInputDeviceAdded()
	@Override
	public void onInputDeviceAdded(int deviceId) {
		// TODO Auto-generated method stub
		InputDeviceState state = getInputDeviceState(deviceId);
		Log.i(TAG, "Device added: " + state.mDevice);
		ResetLabels();
	}
	// Implementation of InputManager.InputDeviceListener.onInputDeviceChanged()
	@Override
	public void onInputDeviceChanged(int deviceId) {
		// TODO Auto-generated method stub
		InputDeviceState state = mInputDeviceStates.get(deviceId);
		if (state != null) {
			mInputDeviceStates.remove(deviceId);
			state = getInputDeviceState(deviceId);
			Log.i(TAG, "Device changed: " + state.mDevice);
		}
		ResetLabels();
	}
	// Implementation of InputManager.InputDeviceListener.onInputDeviceRemoved()
	@Override
	public void onInputDeviceRemoved(int deviceId) {
		// TODO Auto-generated method stub
		InputDeviceState state = mInputDeviceStates.get(deviceId);
		if (state != null) {
			Log.i(TAG, "Device removed: " + state.mDevice);
			mInputDeviceStates.remove(deviceId);
		}
		ResetLabels();
	}

	private static boolean isJoystick(int source) {
		return (source & InputDevice.SOURCE_CLASS_JOYSTICK) != 0;
	}

	/**
	 * Tracks the state of joystick axes and game controller buttons for a particular
	 * input device for diagnostic purposes.
	 */
	private static class InputDeviceState {
		private final InputDevice mDevice;
		private final int[] mAxes;
		private final float[] mAxisValues;
		private final SparseIntArray mKeys;

		public InputDeviceState(InputDevice device) {
			mDevice = device;

			int numAxes = 0;
			final List<MotionRange> ranges = device.getMotionRanges();
			for (MotionRange range : ranges) {
				if ((range.getSource() & InputDevice.SOURCE_CLASS_JOYSTICK) != 0) {
					numAxes += 1;
				}
			}

			mAxes = new int[numAxes];
			mAxisValues = new float[numAxes];
			int i = 0;
			for (MotionRange range : ranges) {
				if ((range.getSource() & InputDevice.SOURCE_CLASS_JOYSTICK) != 0) {
					mAxes[i++] = range.getAxis();
				}
			}

			mKeys = new SparseIntArray();
		}

		public InputDevice getDevice() {
			return mDevice;
		}

		public int getAxisCount() {
			return mAxes.length;
		}

		public int getAxis(int axisIndex) {
			return mAxes[axisIndex];
		}

		public float getAxisValue(int axisIndex) {
			return mAxisValues[axisIndex];
		}

		public int getKeyCount() {
			return mKeys.size();
		}

		public int getKeyCode(int keyIndex) {
			return mKeys.keyAt(keyIndex);
		}

		public boolean isKeyPressed(int keyIndex) {
			return mKeys.valueAt(keyIndex) != 0;
		}

		public boolean onKeyDown(KeyEvent event) {
			final int keyCode = event.getKeyCode();
			if (isGameKey(keyCode)) {
				if (event.getRepeatCount() == 0) {
					final String symbolicName = KeyEvent.keyCodeToString(keyCode);
					mKeys.put(keyCode, 1);
					Log.i(TAG, mDevice.getName() + " - Key Down: " + symbolicName);
				}
				return true;
			}
			return false;
		}

		public boolean onKeyUp(KeyEvent event) {
			final int keyCode = event.getKeyCode();
			if (isGameKey(keyCode)) {
				int index = mKeys.indexOfKey(keyCode);
				if (index >= 0) {
					final String symbolicName = KeyEvent.keyCodeToString(keyCode);
					mKeys.put(keyCode, 0);
					Log.i(TAG, mDevice.getName() + " - Key Up: " + symbolicName);
				}
				return true;
			}
			return false;
		}

		public boolean onJoystickMotion(MotionEvent event) {
			StringBuilder message = new StringBuilder();
			message.append(mDevice.getName()).append(" - Joystick Motion:\n");

			final int historySize = event.getHistorySize();
			for (int i = 0; i < mAxes.length; i++) {
				final int axis = mAxes[i];
				final float value = event.getAxisValue(axis);
				mAxisValues[i] = value;
				message.append("  ").append(MotionEvent.axisToString(axis)).append(": ");

				// Append all historical values in the batch.
				for (int historyPos = 0; historyPos < historySize; historyPos++) {
					message.append(event.getHistoricalAxisValue(axis, historyPos));
					message.append(", ");
				}

				// Append the current value.
				message.append(value);
				message.append("\n");
			}
			Log.i(TAG, message.toString());
			return true;
		}

		// Check whether this is a key we care about.
		// In a real game, we would probably let the user configure which keys to use
		// instead of hardcoding the keys like this.
		private static boolean isGameKey(int keyCode) {
			switch (keyCode) {
			case KeyEvent.KEYCODE_DPAD_UP:
			case KeyEvent.KEYCODE_DPAD_DOWN:
			case KeyEvent.KEYCODE_DPAD_LEFT:
			case KeyEvent.KEYCODE_DPAD_RIGHT:
			case KeyEvent.KEYCODE_DPAD_CENTER:
			case KeyEvent.KEYCODE_SPACE:
				return true;
			default:
				return KeyEvent.isGamepadButton(keyCode);
			}
		}
	}


	//display results
	public void show(InputDeviceState state) {

		//final int BASE_ID_AXIS_ITEM = 3 << 10;
		//final int BASE_ID_KEY_ITEM = 4 << 10;

		TextView textView = (TextView) this.findViewById(R.id.game_controller_input_heading);
		//int source=state.getDevice().getSources();
		//String ssources=Integer.toString(source);
		textView.setText(state.getDevice().getName());
		int dcolor=textView.getCurrentTextColor();

		// Populate axes.
		final int axisCount = state.getAxisCount();
		//String temp="Axis: ";
		//textView = (TextView) this.findViewById(R.id.game_controller_axis_x);
		for (int i = 0; i < axisCount; i++) {
			final int axis = state.getAxis(i);
			//final int id = BASE_ID_AXIS_ITEM | axis;
			//temp=temp+Integer.toString(id)+" "+MotionEvent.axisToString(axis)+"= "+Float.toString(state.getAxisValue(i))+", ";
			textView=null;
			switch(axis){
			case MotionEvent.AXIS_X:
				textView = (TextView) this.findViewById(R.id.game_controller_axis_x);
				
				break;
			case MotionEvent.AXIS_Y:
				textView = (TextView) this.findViewById(R.id.game_controller_axis_y);
				
				break;
			case MotionEvent.AXIS_Z:
				textView = (TextView) this.findViewById(R.id.game_controller_axis_z);
				
				break;
			case MotionEvent.AXIS_RZ:
				textView = (TextView) this.findViewById(R.id.game_controller_axis_rz);
				
				break;
			case MotionEvent.AXIS_HAT_X:
				textView = (TextView) this.findViewById(R.id.game_controller_axis_hat_x);
				
				break;
			case MotionEvent.AXIS_HAT_Y:
				textView = (TextView) this.findViewById(R.id.game_controller_axis_hat_y);
				
				break;
			}
			if (textView!=null){
				textView.setTextColor(Color.RED);
				if (state.getAxisValue(i)==0.0){
					textView.setTextColor(dcolor);
				}
				textView.setText(MotionEvent.axisToString(axis)+"= "+Float.toString(state.getAxisValue(i)));
			}
			
		}
		//textView.setText(temp);    

		// Populate keys.
		final int keyCount = state.getKeyCount();
		//temp="Buttons: ";
		//textView = (TextView) this.findViewById(R.id.game_controller_b1);
		
		for (int i = 0; i < keyCount; i++) {
			final int keyCode = state.getKeyCode(i);
			//final int id = BASE_ID_KEY_ITEM | keyCode;
			textView=null;
			switch(state.getKeyCode(i)){
			case KeyEvent.KEYCODE_BUTTON_1:
				textView = (TextView) this.findViewById(R.id.game_controller_b1);
				break;
			case KeyEvent.KEYCODE_BUTTON_2:
				textView = (TextView) this.findViewById(R.id.game_controller_b2);
				break;
			case KeyEvent.KEYCODE_BUTTON_3:
				textView = (TextView) this.findViewById(R.id.game_controller_b3);
				break;
			case KeyEvent.KEYCODE_BUTTON_4:
				textView = (TextView) this.findViewById(R.id.game_controller_b4);
				break;
			case KeyEvent.KEYCODE_BUTTON_5:
				textView = (TextView) this.findViewById(R.id.game_controller_b5);
				break;
			case KeyEvent.KEYCODE_BUTTON_6:
				textView = (TextView) this.findViewById(R.id.game_controller_b6);
				break;
			case KeyEvent.KEYCODE_BUTTON_7:
				textView = (TextView) this.findViewById(R.id.game_controller_b7);
				break;
			case KeyEvent.KEYCODE_BUTTON_8:
				textView = (TextView) this.findViewById(R.id.game_controller_b8);
				break;
			case KeyEvent.KEYCODE_BUTTON_9:
				textView = (TextView) this.findViewById(R.id.game_controller_b9);
				break;
			case KeyEvent.KEYCODE_BUTTON_10:
				textView = (TextView) this.findViewById(R.id.game_controller_b10);
				break;
			case KeyEvent.KEYCODE_BUTTON_11:
				textView = (TextView) this.findViewById(R.id.game_controller_b11);
				break;
			case KeyEvent.KEYCODE_BUTTON_12:
				textView = (TextView) this.findViewById(R.id.game_controller_b12);
				break;
			case KeyEvent.KEYCODE_BUTTON_13:
				textView = (TextView) this.findViewById(R.id.game_controller_b13);
				break;
			case KeyEvent.KEYCODE_BUTTON_14:
				textView = (TextView) this.findViewById(R.id.game_controller_b14);
				break;
			case KeyEvent.KEYCODE_BUTTON_15:
				textView = (TextView) this.findViewById(R.id.game_controller_b15);
				break;
			case KeyEvent.KEYCODE_BUTTON_16:
				textView = (TextView) this.findViewById(R.id.game_controller_b16);
				break;
			case KeyEvent.KEYCODE_DPAD_RIGHT:
				textView = (TextView) this.findViewById(R.id.game_controller_right);
				break;
			case KeyEvent.KEYCODE_DPAD_LEFT:
				textView = (TextView) this.findViewById(R.id.game_controller_left);
				break;
			case KeyEvent.KEYCODE_DPAD_UP:
				textView = (TextView) this.findViewById(R.id.game_controller_up);
				break;
			case KeyEvent.KEYCODE_DPAD_DOWN:
				textView = (TextView) this.findViewById(R.id.game_controller_down);
				break;
			}
			if (textView!=null){
				//String temp2="";
				textView.setTextColor(dcolor);
				if (state.isKeyPressed(i)==true){
					//temp2=" down";
					textView.setTextColor(Color.RED);
				}
				textView.setText(KeyEvent.keyCodeToString(keyCode));
				//textView.setText(Integer.toString(keyCode));
			}
			//temp=temp+Integer.toString(id) + " " + KeyEvent.keyCodeToString(keyCode) +" "+temp2+", ";

		}
		//textView.setText(temp); 


	}
	
	public void ResetLabels(){
		TextView textView = (TextView) this.findViewById(R.id.game_controller_input_heading);
		int dcolor=textView.getCurrentTextColor();
		textView.setText(getString(R.string.game_controller_input_description));
	
		textView = (TextView) this.findViewById(R.id.game_controller_axis_x);
		textView.setText(getString(R.string.game_controller_axis_x));
		textView.setTextColor(dcolor);
		
		textView = (TextView) this.findViewById(R.id.game_controller_axis_y);
		textView.setText(getString(R.string.game_controller_axis_y));
		textView.setTextColor(dcolor);
		
		textView = (TextView) this.findViewById(R.id.game_controller_axis_z);
		textView.setText(getString(R.string.game_controller_axis_z));
		textView.setTextColor(dcolor);
		
		textView = (TextView) this.findViewById(R.id.game_controller_axis_rz);
		textView.setText(getString(R.string.game_controller_axis_rz));
		textView.setTextColor(dcolor);
		
		textView = (TextView) this.findViewById(R.id.game_controller_axis_hat_x);
		textView.setText(getString(R.string.game_controller_axis_hat_x));
		textView.setTextColor(dcolor);
		
		textView = (TextView) this.findViewById(R.id.game_controller_axis_hat_y);
		textView.setText(getString(R.string.game_controller_axis_hat_y));
		textView.setTextColor(dcolor);
		
		textView = (TextView) this.findViewById(R.id.game_controller_b1);
		textView.setText(getString(R.string.game_controller_b1));
		textView.setTextColor(dcolor);
		
		textView = (TextView) this.findViewById(R.id.game_controller_b2);
		textView.setText(getString(R.string.game_controller_b2));
		textView.setTextColor(dcolor);
		
		textView = (TextView) this.findViewById(R.id.game_controller_b3);
		textView.setText(getString(R.string.game_controller_b3));
		textView.setTextColor(dcolor);
		
		textView = (TextView) this.findViewById(R.id.game_controller_b4);
		textView.setText(getString(R.string.game_controller_b4));
		textView.setTextColor(dcolor);
		
		textView = (TextView) this.findViewById(R.id.game_controller_b5);
		textView.setText(getString(R.string.game_controller_b5));
		textView.setTextColor(dcolor);
		
		textView = (TextView) this.findViewById(R.id.game_controller_b6);
		textView.setText(getString(R.string.game_controller_b6));
		textView.setTextColor(dcolor);
		
		textView = (TextView) this.findViewById(R.id.game_controller_b7);
		textView.setText(getString(R.string.game_controller_b7));
		textView.setTextColor(dcolor);
		
		textView = (TextView) this.findViewById(R.id.game_controller_b8);
		textView.setText(getString(R.string.game_controller_b8));
		textView.setTextColor(dcolor);
		
		textView = (TextView) this.findViewById(R.id.game_controller_b9);
		textView.setText(getString(R.string.game_controller_b9));
		textView.setTextColor(dcolor);
		
		textView = (TextView) this.findViewById(R.id.game_controller_b10);
		textView.setText(getString(R.string.game_controller_b10));
		textView.setTextColor(dcolor);
		
		textView = (TextView) this.findViewById(R.id.game_controller_b11);
		textView.setText(getString(R.string.game_controller_b11));
		textView.setTextColor(dcolor);
		
		textView = (TextView) this.findViewById(R.id.game_controller_b12);
		textView.setText(getString(R.string.game_controller_b12));
		textView.setTextColor(dcolor);
		
		textView = (TextView) this.findViewById(R.id.game_controller_b13);
		textView.setText(getString(R.string.game_controller_b13));
		textView.setTextColor(dcolor);
		
		textView = (TextView) this.findViewById(R.id.game_controller_b14);
		textView.setText(getString(R.string.game_controller_b14));
		textView.setTextColor(dcolor);
		
		textView = (TextView) this.findViewById(R.id.game_controller_b15);
		textView.setText(getString(R.string.game_controller_b15));
		textView.setTextColor(dcolor);
		
		textView = (TextView) this.findViewById(R.id.game_controller_b16);
		textView.setText(getString(R.string.game_controller_b16));
		textView.setTextColor(dcolor);
		
		
	}

}
